<?php

namespace Drupal\acquia_search\v2;

use Drupal\acquia_search\AcquiaSearchSolrApiBase;
use Drupal\acquia_search\AcquiaSearchSolrApiInterface;

/**
 * Class AcquiaSearchApi.
 *
 * This class is used to connect Drupal to the Search clusters. It is agnostic
 * from the Search platform (Search API / ApacheSolr)
 */
class AcquiaSearchSolrApi extends AcquiaSearchSolrApiBase implements AcquiaSearchSolrApiInterface {

  /**
   * Define the Acquia Search Version.
   */
  const ACQUIA_SEARCH_VERSION = 2;

  /**
   * Wrapper for constructor.
   *
   * @param string $subscription
   *   Subscription name.
   * @param string $api_key
   *   API key.
   * @param string $host
   *   Search host name.
   * @param string $uuid
   *   Application UUID.
   * @param string $subscription_data
   *   Subscription Data.
   *
   * @return \Drupal\acquia_search\AcquiaSearchSolrApiInterface
   *   API instance.
   *
   * @throws \InvalidArgumentException
   */
  public static function get($subscription, $api_key, $host, $uuid, $subscription_data = NULL) {
    // Redeclare the version in static context.
    if (empty($subscription) || empty($api_key) || empty($host) || empty($uuid)) {
      throw new \InvalidArgumentException('Please provide API credentials for Acquia Search.');
    }

    return new self($subscription, $api_key, $host, $uuid, $subscription_data);
  }

  /**
   * Get API from settings.
   *
   * @return \Drupal\acquia_search\AcquiaSearchSolrApiInterface
   *   API instance.
   *
   * @throws \InvalidArgumentException
   */
  public static function getFromSettings() {
    $api_key = acquia_agent_settings('acquia_key');
    // If no UUID explicitly set, get it from subscription data.
    $uuid = variable_get('acquia_uuid', acquia_agent_settings('acquia_subscription_data')['uuid']);

    if (!empty($conf['acquia_subscription_id']) && !empty($conf['acquia_subscription_key'])) {
      $identifier = $conf['acquia_subscription_id'];
      $key = $conf['acquia_subscription_key'];
      $subscription = acquia_agent_get_subscription($params = [], $identifier, $key);
    }
    else {
      $identifier = acquia_agent_settings('acquia_identifier');
      $subscription = acquia_agent_settings('acquia_subscription_data');
    }
    $search_host = variable_get('acquia_search_v' . self::ACQUIA_SEARCH_VERSION . '_host', 'search.acquia.com');
    // @todo rework this logic once we have AH_SERVER_REGION
    // legacy_internal will be true if 'internal-' is the start of the host
    // name that's set in via Acquia Cloud platform or vset.
    $legacy_internal = preg_match('/internal[-.]/', $search_host);
    // Adding the subscription specific colony to the heartbeat data.
    if (!empty($subscription['heartbeat_data']['search_service_colony'])) {
      $search_host = $subscription['heartbeat_data']['search_service_colony'];
      if ($legacy_internal) {
        // If we want to be using an internal host name, fix the one
        // from the hearbeat data here.
        $search_host = 'internal-' . $search_host;
      }
    }
    // Check if we are on Acquia Cloud hosting. @see NN-2503.
    if (!empty($_ENV['AH_SITE_ENVIRONMENT']) && !empty($_ENV['AH_SERVER_REGION'])) {
      if ($_ENV['AH_SERVER_REGION'] == 'us-east-1' && $search_host == 'search.acquia.com') {
        $search_host = 'internal-search.acquia.com';
      }
      elseif (strpos($search_host, 'search-' . $_ENV['AH_SERVER_REGION']) === 0) {
        $search_host = 'internal-' . $search_host;
      }
    }

    if (empty($subscription) || empty($api_key) || empty($search_host) || empty($uuid)) {
      throw new \InvalidArgumentException('Please provide API credentials for Acquia Search.');
    }

    return new self($identifier, $api_key, $search_host, $uuid, $subscription);
  }

  /**
   * Returns list of search cores.
   *
   * Note: This is one of the critical differences between v2 and v3. Search
   * cores are retrieved from the subscription instead of directly from Search.
   *
   * @return array
   *   Search cores list.
   */
  public function getCores() {
    $result = [];
    $cores = $this->subscriptionData['heartbeat_data']['search_cores'];

    if (is_array($cores)) {
      foreach ($cores as $core) {
        $result[$core['core_id']] = [
          'host' => $core['balancer'],
          'core_id' => $core['core_id'],
          'data' => $core,
        ];
      }
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getUrl($server_id) {
    return 'https://' . $this->host . variable_get('acquia_search_path', '/solr/' . $this->subscription);
  }

  /**
   * {@inheritdoc}
   */
  public function getServiceClass($ecosystem) {
    return 'AcquiaSearchV2' . $ecosystem;
  }

  /**
   * {@inheritdoc}
   */
  public function getPossibleCores(&$possible_cores) {
    // For production-only, we allow auto-connecting to the suffix-less core
    // as the fallback.
    if (isset($_SERVER['AH_PRODUCTION']) || isset($_ENV['AH_PRODUCTION'])) {
      $possible_cores[] = $this->subscription;
    }
  }

}
